@@ -19,7 +19,7 @@
 // Modified by Garth N. Wells, 2010-2011.
 //
 // First added:  2008-12-12
-// Last changed: 2011-01-22
+// Last changed: 2012-11-12
 //
 // This demo illustrates basic usage of block matrices and vectors.
 
@@ -31,17 +31,17 @@ using namespace dolfin;
 int main()
 {
   // Create mesh
-  UnitSquare mesh(32, 32);
+  auto mesh = std::make_shared<UnitSquareMesh>(32, 32);
 
   // Create a simple stiffness matrix and vector
-  StiffnessMatrix::FunctionSpace V(mesh);
+  auto V = std::make_shared<StiffnessMatrix::FunctionSpace>(mesh);
 
   StiffnessMatrix::BilinearForm a(V, V);
-  boost::shared_ptr<GenericMatrix> A(new Matrix);
+  std::shared_ptr<GenericMatrix> A(new Matrix);
   assemble(*A, a);
 
   StiffnessMatrix::LinearForm L(V);
-  boost::shared_ptr<GenericVector> x(new Vector);
+  std::shared_ptr<GenericVector> x(new Vector);
   assemble(*x, L);
 
   // Create a block matrix
@@ -57,8 +57,8 @@ int main()
   xx.set_block(1, x);
 
   // Create another block vector
-  boost::shared_ptr<GenericVector> y(new Vector);
-  A->resize(*y, 0);
+  std::shared_ptr<GenericVector> y(new Vector);
+  A->init_vector(*y, 0);
   BlockVector yy(2);
   yy.set_block(0, y);
   yy.set_block(1, y);
