@@ -134,9 +134,5 @@ int main()
   File file("poisson.pvd");
   file << u;
 
-  // Plot solution
-  plot(u);
-  interactive();
-
   return 0;
 }
