@@ -14,9 +14,6 @@
 //
 // You should have received a copy of the GNU Lesser General Public License
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
-//
-// First added:  2011-06-22
-// Last changed: 2012-08-20
 
 #ifndef __LINEAR_VARIATIONAL_PROBLEM_H
 #define __LINEAR_VARIATIONAL_PROBLEM_H
@@ -46,29 +43,12 @@ namespace dolfin
   {
   public:
 
-    /// Create linear variational problem without boundary conditions
-    LinearVariationalProblem(const Form& a,
-                             const Form& L,
-                             Function& u);
-
-    /// Create linear variational problem with a single boundary condition
-    LinearVariationalProblem(const Form& a,
-                             const Form& L,
-                             Function& u,
-                             const DirichletBC& bc);
-
-    /// Create linear variational problem with a list of boundary conditions
-    LinearVariationalProblem(const Form& a,
-                             const Form& L,
-                             Function& u,
-                             std::vector<const DirichletBC*> bcs);
-
-    /// Create linear variational problem with a list of boundary conditions
-    /// (shared pointer version)
+    /// Create linear variational problem with a list of boundary
+    /// conditions
     LinearVariationalProblem(std::shared_ptr<const Form> a,
                              std::shared_ptr<const Form> L,
                              std::shared_ptr<Function> u,
-                             std::vector<std::shared_ptr<const DirichletBC> > bcs);
+                             std::vector<std::shared_ptr<const DirichletBC>> bcs);
 
     /// Return bilinear form
     std::shared_ptr<const Form> bilinear_form() const;
@@ -83,7 +63,7 @@ namespace dolfin
     std::shared_ptr<const Function> solution() const;
 
     /// Return boundary conditions
-    std::vector<std::shared_ptr<const DirichletBC> > bcs() const;
+    std::vector<std::shared_ptr<const DirichletBC>> bcs() const;
 
     /// Return trial space
     std::shared_ptr<const FunctionSpace> trial_space() const;
@@ -106,7 +86,7 @@ namespace dolfin
     std::shared_ptr<Function> _u;
 
     // The Dirichlet boundary conditions
-    std::vector<std::shared_ptr<const DirichletBC> > _bcs;
+    std::vector<std::shared_ptr<const DirichletBC>> _bcs;
 
   };
 
