@@ -1,4 +1,4 @@
-// Copyright (C) 2008-2013 Kent-Andre Mardal and Garth N. Wells
+// Copyright (C) 2008-2015 Kent-Andre Mardal and Garth N. Wells
 //
 // This file is part of DOLFIN.
 //
@@ -16,9 +16,6 @@
 // along with DOLFIN. If not, see <http://www.gnu.org/licenses/>.
 //
 // Modified by Anders Logg 2008-2011
-//
-// First added:  2009-06-22
-// Last changed: 2013-04-23
 
 #ifndef __SYSTEM_ASSEMBLER_H
 #define __SYSTEM_ASSEMBLER_H
@@ -53,8 +50,8 @@ namespace dolfin
   template<typename T> class MeshFunction;
   class UFC;
 
-  /// This class provides an assembler for systems of the form
-  /// Ax = b. It differs from the default DOLFIN assembler in that it
+  /// This class provides an assembler for systems of the form Ax =
+  /// b. It differs from the default DOLFIN assembler in that it
   /// applies boundary conditions at the time of assembly, which
   /// preserves any symmetries in A.
 
@@ -62,29 +59,10 @@ namespace dolfin
   {
   public:
 
-    /// Constructor
-    SystemAssembler(const Form& a, const Form& L);
-
-    /// Constructor
-    SystemAssembler(const Form& a, const Form& L, const DirichletBC& bc);
-
-    /// Constructor
-    SystemAssembler(const Form& a, const Form& L,
-                    const std::vector<const DirichletBC*> bcs);
-
-    /// Constructor
-    SystemAssembler(std::shared_ptr<const Form> a,
-                    std::shared_ptr<const Form> L);
-
     /// Constructor
     SystemAssembler(std::shared_ptr<const Form> a,
                     std::shared_ptr<const Form> L,
-                    const DirichletBC& bc);
-
-    /// Constructor
-    SystemAssembler(std::shared_ptr<const Form> a,
-                    std::shared_ptr<const Form> L,
-                    const std::vector<const DirichletBC*> bcs);
+                    std::vector<std::shared_ptr<const DirichletBC>> bcs);
 
     /// Assemble system (A, b)
     void assemble(GenericMatrix& A, GenericVector& b);
@@ -120,6 +98,10 @@ namespace dolfin
     static void check_arity(std::shared_ptr<const Form> a,
                             std::shared_ptr<const Form> L);
 
+    // Check if _bcs[bc_index] is part of FunctionSpace fs
+    bool check_functionspace_for_bc
+      (std::shared_ptr<const FunctionSpace> fs, std::size_t bc_index);
+
     // Assemble system
     void assemble(GenericMatrix* A, GenericVector* b,
                   const GenericVector* x0);
@@ -128,13 +110,13 @@ namespace dolfin
     std::shared_ptr<const Form> _a, _l;
 
     // Boundary conditions
-    std::vector<const DirichletBC*> _bcs;
+    std::vector<std::shared_ptr<const DirichletBC>> _bcs;
 
     static void cell_wise_assembly(
       std::array<GenericTensor*, 2>& tensors,
       std::array<UFC*, 2>& ufc,
       Scratch& data,
-      const DirichletBC::Map& boundary_values,
+      const std::vector<DirichletBC::Map>& boundary_values,
       std::shared_ptr<const MeshFunction<std::size_t>> cell_domains,
       std::shared_ptr<const MeshFunction<std::size_t>> exterior_facet_domains);
 
@@ -142,7 +124,7 @@ namespace dolfin
       std::array<GenericTensor*, 2>& tensors,
       std::array<UFC*, 2>& ufc,
       Scratch& data,
-      const DirichletBC::Map& boundary_values,
+      const std::vector<DirichletBC::Map>& boundary_values,
       std::shared_ptr<const MeshFunction<std::size_t>> cell_domains,
       std::shared_ptr<const MeshFunction<std::size_t>> exterior_facet_domains,
       std::shared_ptr<const MeshFunction<std::size_t>> interior_facet_domains);
@@ -153,7 +135,7 @@ namespace dolfin
       std::array<std::vector<double>, 2>& Ae,
       std::array<UFC*, 2>& ufc,
       ufc::cell& ufc_cell,
-      std::vector<double>& vertex_coordinates,
+      std::vector<double>& coordinate_dofs,
       const std::array<bool, 2>& tensor_required_cell,
       const std::array<bool, 2>& tensor_required_facet,
       const Cell& cell,
@@ -167,7 +149,7 @@ namespace dolfin
     static void compute_interior_facet_tensor(
       std::array<UFC*, 2>& ufc,
       std::array<ufc::cell, 2>& ufc_cell,
-      std::array<std::vector<double>, 2>& vertex_coordinates,
+      std::array<std::vector<double>, 2>& coordinate_dofs,
       const std::array<bool, 2>& tensor_required_cell,
       const std::array<bool, 2>& tensor_required_facet,
       const std::array<Cell, 2>& cell,
@@ -189,7 +171,7 @@ namespace dolfin
       const std::array<std::vector<ArrayView<const la_index>>, 2>& cell_dofs);
 
     static void apply_bc(double* A, double* b,
-                         const DirichletBC::Map& boundary_values,
+                         const std::vector<DirichletBC::Map>& boundary_values,
                          const ArrayView<const dolfin::la_index>& global_dofs0,
                          const ArrayView<const dolfin::la_index>& global_dofs1);
 
@@ -202,7 +184,7 @@ namespace dolfin
     static bool
       cell_matrix_required(const GenericTensor* A,
                            const void* integral,
-                           const DirichletBC::Map& boundary_values,
+                           const std::vector<DirichletBC::Map>& boundary_values,
                            const ArrayView<const dolfin::la_index>& dofs);
 
   };
