@@ -30,8 +30,6 @@
 #include <dolfin/fem/MultiMeshDirichletBC.h>
 #include <dolfin/fem/MultiMeshDofMap.h>
 #include <dolfin/fem/MultiMeshForm.h>
-
-// Move up when ready or merge with Assembler.h
-#include <dolfin/fem/OpenMpAssembler.h>
+#include <dolfin/fem/PETScDMCollection.h>
 
 #endif
